"""
Model Selection Utilities for ML Builder.

This package contains utility functions for model selection including:
- Model characteristics and properties
- Recommendation strategies
- Compatibility checking utilities
"""